@@ -33,6 +33,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.ObjectInputStream;
 import java.util.Locale;
+import java.util.Random;
 
 import javax.swing.JButton;
 import javax.swing.JCheckBoxMenuItem;
@@ -50,45 +51,59 @@ import javax.swing.border.EtchedBorder;
 import javax.swing.border.TitledBorder;
 
 import jfractionlab.displays.PointDisplay;
-import jfractionlab.excerciseDialogs.ClickNumerator;
-import jfractionlab.excerciseDialogs.CompareFractions;
-import jfractionlab.excerciseDialogs.DivideFractionsByFractions;
-import jfractionlab.excerciseDialogs.DivideFractionsByNumbers;
-import jfractionlab.excerciseDialogs.DivideNumbersByFractions;
-import jfractionlab.excerciseDialogs.ExtendFraction;
-import jfractionlab.excerciseDialogs.ImproperFraction;
-import jfractionlab.excerciseDialogs.MixedNumbers;
-import jfractionlab.excerciseDialogs.MultiplyFractions;
-import jfractionlab.excerciseDialogs.NameFraction;
-import jfractionlab.excerciseDialogs.PlusAndMinus;
-import jfractionlab.excerciseDialogs.ReduceFraction;
+import jfractionlab.exerciseDialogs.ClickNumerator;
+import jfractionlab.exerciseDialogs.CompareFractions;
+import jfractionlab.exerciseDialogs.DivideFractionsByFractions;
+import jfractionlab.exerciseDialogs.DivideFractionsByNumbers;
+import jfractionlab.exerciseDialogs.DivideNumbersByFractions;
+import jfractionlab.exerciseDialogs.ExtendFraction;
+import jfractionlab.exerciseDialogs.ImproperFraction;
+import jfractionlab.exerciseDialogs.MixedNumbers;
+import jfractionlab.exerciseDialogs.MultiplyFractions;
+import jfractionlab.exerciseDialogs.NameFraction;
+import jfractionlab.exerciseDialogs.PlusAndMinus;
+import jfractionlab.exerciseDialogs.ReduceFraction;
+import jfractionlab.jflOptionDialogs.AdditionWorkSheetDialog;
 import jfractionlab.jflOptionDialogs.InfoDialog;
 import jfractionlab.jflOptionDialogs.LogSaverDialog;
+import jfractionlab.jflOptionDialogs.MultiplicationWorksheetDialog;
 
-//fixme jfractionlab.java
 
-//todo jfractionlab.java
+
+//TODO jfractionlab.JFractionLab
 //
-// eigene dialoge besser strukturieren!!
+// worksheets als pdf 
+// pdflatex fuer windows
+// eigene aufgaben beim kuerzen
+// worksheets als *odt"
+// neuer aufgabentyp: vergleichen durch gleichnamig machen
+// "fraction-calculator"
+// spiel 1 und 2 : auch "falsche pizzen" anzeigen und auch andere formen anzeigen
 
-// optionen speichern (kuerzen, eigene aufgaben)
-// spiel 1 und 2 : auch "falsche pizzen" anzeigen und auch andere formen
-//
-//seminarix/debian, knoppix, opensuse, heise?
-//javadoc
+//document the code with javadoc
 
 public class JFractionLab extends JFrame implements ActionListener, ItemListener{
 	public static final long serialVersionUID = 54L;
 	public static final String jep_fontface="<font face='Avalon,Wide Latin'>";
+	public static final Random ran = new Random(System.currentTimeMillis());
 	private JMenuBar jmb = new JMenuBar();
 	private JMenu jmLanguage = new JMenu("");
-	private JMenuItem jmiGerman = new JMenuItem("");
-	private JMenuItem jmiFrench = new JMenuItem("");
-	private JMenuItem jmiSpanish = new JMenuItem("");
-	private JMenuItem jmiEnglish = new JMenuItem("");
-	private JMenuItem jmiItalian = new JMenuItem("");
-	private JMenuItem jmiPortuguese = new JMenuItem("");
-//	private JMenuItem jmiPolish = new JMenuItem("");
+		private JMenuItem jmiGerman = new JMenuItem("");
+		private JMenuItem jmiFrench = new JMenuItem("");
+		private JMenuItem jmiSpanish = new JMenuItem("");
+		private JMenuItem jmiEnglish = new JMenuItem("");
+		private JMenuItem jmiItalian = new JMenuItem("");
+		private JMenuItem jmiPortuguese = new JMenuItem("");
+//		private JMenuItem jmiPolish = new JMenuItem("");
+	private JMenu jmWorkSheet = new JMenu("");
+//		private JMenuItem jmiExtendingWorkSheet = new JMenuItem("");
+//		private JMenuItem jmiReducingWorkSheet = new JMenuItem("");
+//		private JMenuItem jmiImproperFractionsWorkSheet = new JMenuItem("");
+//		private JMenuItem jmiMixedNumbersWorkSheet = new JMenuItem("");
+		private JMenuItem jmiAdditionWorkSheet = new JMenuItem("");
+//		private JMenuItem jmiSubtractionWorkSheet = new JMenuItem("");
+		private JMenuItem jmiMultiplicationWorkSheet = new JMenuItem("");
+//		private JMenuItem jmiDivisionWorkSheet = new JMenuItem("");
 	private JMenu jmHelp = new JMenu("");
 	private static JCheckBoxMenuItem cb_showtippatstart = new JCheckBoxMenuItem();
 	private JMenuItem jmiAbout = new JMenuItem("");
@@ -101,8 +116,8 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 	private PointDisplay dsp_nameFraction = new PointDisplay(true);
 	private JButton btn_compareFractions;//Vergleiche_Pizzen
 	private PointDisplay dsp_compareFractions = new PointDisplay(true);
-	private JButton btn_expandFraction;
-	private PointDisplay dsp_expandFraction = new PointDisplay(true);
+	private JButton btn_extendFraction;
+	private PointDisplay dsp_extendFraction = new PointDisplay(true);
 	private JButton btn_reduceFraction;
 	private PointDisplay dsp_reduceFraction = new PointDisplay(true);
 	private JButton btn_improperFracions;
@@ -134,7 +149,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 	public int points_nameFraction = 0;
 	public int points_compareFractions = 0;
 	public int points_compareInvisibleFractions = 0;
-	public int points_expandFraction = 0;
+	public int points_extendFraction = 0;
 	public int points_reduceFraction = 0;
 	public int points_unechteBrueche = 0;
 	public int points_gemischteZahlen = 0;
@@ -148,7 +163,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 	//Damit alle Dialoge dieselbe Schrift haben, ist sie hier static definiert.
 	public static Font infofont = new Font("SansSerif",Font.BOLD,24);
 	public static final int nbOfCols = 16;
-	
+	private String aboutText = "";
 	
 	/**
 	* Create the GUI and show it.
@@ -160,20 +175,11 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 	public static void main(String[] args) {
 		javax.swing.SwingUtilities.invokeLater(new Runnable() {
 			public void run() {
-				//new JFractionLab();
-				createAndShowGUI();
+				new JFractionLab();
 			}//run
 		});//invokeLater
 	}
 	
-	/**
-	 * 
-	 *
-	 */
-	private static void createAndShowGUI() {
-		new JFractionLab();
-	}
-	
 	/**
 	 * 
 	 * @throws HeadlessException
@@ -215,6 +221,40 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		jmiPortuguese.addActionListener(this);
 		jmLanguage.add(jmiPortuguese);
 		
+		jmWorkSheet.setText(lang.Messages.getString("worksheets"));
+
+//		jmiExtendingWorkSheet.addActionListener(this);
+//		jmiExtendingWorkSheet.setText(lang.Messages.getString("extend_fractions"));
+//		jmWorkSheet.add(jmiExtendingWorkSheet);
+//		
+//		jmiReducingWorkSheet.addActionListener(this);
+//		jmiReducingWorkSheet.setText(lang.Messages.getString("reduce_fractions"));
+//		jmWorkSheet.add(jmiReducingWorkSheet);
+//		
+//		jmiImproperFractionsWorkSheet.addActionListener(this);
+//		jmiImproperFractionsWorkSheet.setText(lang.Messages.getString("improper_fracs"));
+//		jmWorkSheet.add(jmiImproperFractionsWorkSheet);
+//		
+//		jmiMixedNumbersWorkSheet.addActionListener(this);
+//		jmiMixedNumbersWorkSheet.setText(lang.Messages.getString("mixed_numbers"));
+//		jmWorkSheet.add(jmiMixedNumbersWorkSheet);
+//		
+		jmiAdditionWorkSheet.addActionListener(this);
+		jmiAdditionWorkSheet.setText(lang.Messages.getString("add_fractions"));
+		jmWorkSheet.add(jmiAdditionWorkSheet);
+//		
+//		jmiSubtractionWorkSheet.addActionListener(this);
+//		jmiSubtractionWorkSheet.setText(lang.Messages.getString("minus_fractions"));
+//		jmWorkSheet.add(jmiSubtractionWorkSheet);
+//
+		jmiMultiplicationWorkSheet.addActionListener(this);
+		jmiMultiplicationWorkSheet.setText(lang.Messages.getString("multiply_fractions"));
+		jmWorkSheet.add(jmiMultiplicationWorkSheet);
+//		
+//		jmiDivisionWorkSheet.addActionListener(this);
+//		jmiDivisionWorkSheet.setText("neues wort!!");
+//		jmWorkSheet.add(jmiDivisionWorkSheet);
+		
 		jmHelp.setText(lang.Messages.getString("jmi_help"));
 		
 		cb_showtippatstart.setText(lang.Messages.getString("show_hints_on_start"));
@@ -234,6 +274,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		jmHelp.add(jmiAbout);
 				
 		jmb.add(jmLanguage);
+		jmb.add(jmWorkSheet);
 		jmb.add(jmHelp);
 		setJMenuBar(jmb);
 		double sizes_main[][] = {{
@@ -288,10 +329,10 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 			btn_compareFractions.addActionListener(this);
 		pn_oben.add(btn_compareFractions, "0,2,1,2");
 		pn_oben.add(dsp_compareFractions, "2,2");
-			btn_expandFraction = new JButton("4. "+lang.Messages.getString("extend_fractions"));
-			btn_expandFraction.addActionListener(this);
-		pn_oben.add(btn_expandFraction, "0,3,1,3");
-		pn_oben.add(dsp_expandFraction, "2,3");
+			btn_extendFraction = new JButton("4. "+lang.Messages.getString("extend_fractions"));
+			btn_extendFraction.addActionListener(this);
+		pn_oben.add(btn_extendFraction, "0,3,1,3");
+		pn_oben.add(dsp_extendFraction, "2,3");
 			btn_reduceFraction = new JButton("5. "+lang.Messages.getString("reduce_fractions"));
 			btn_reduceFraction.addActionListener(this);
 		pn_oben.add(btn_reduceFraction, "0,4,1,4");
@@ -364,37 +405,32 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		}
 		//setMainWindowPointDisplaysForDemoReason();
 		setVisible(true);
-	}//Constructor
 
-//	private void setMainWindowPointDisplaysForDemoReason(){
-//		points_clickNumerator = 0;
-//		points_nameFraction = 0;
-//		points_compareFractions = 0;
-//		points_compareInvisibleFractions = 0;
-//		points_expandFraction = 0;
-//		points_reduceFraction = 0;
-//		points_unechteBrueche = 0;
-//		points_gemischteZahlen = 0;
-//		points_addFractions = 10;
-//		points_subtractFractions = 10;
-//		points_multiplyFractions = 10;
-//		points_divideFractionsByNumbers = 10;
-//		points_divideNumbersByFractions = 10;
-//		points_divideFractionsByFractions = 10;
-//		dsp_addFractions.setText(String.valueOf(25));
-//		dsp_clickNumerator.setText(String.valueOf(0));
-//		dsp_compareFractions.setText(String.valueOf(3));
-//		dsp_divideFractionsByFractions.setText(String.valueOf(0));
-//		dsp_divideFractionsByNumbers.setText(String.valueOf(0));
-//		dsp_divideNumbersByFractions.setText(String.valueOf(0));
-//		dsp_expandFraction.setText(String.valueOf(0));
-//		dsp_improperFractions.setText(String.valueOf(0));
-//		dsp_mixedNumbers.setText(String.valueOf(0));
-//		dsp_multiplyFractions.setText(String.valueOf(0));
-//		dsp_nameFraction.setText(String.valueOf(14));
-//		dsp_reduceFraction.setText(String.valueOf(0));
-//		dsp_subtractFractions.setText(String.valueOf(15));
-//	}
+		aboutText =
+			lang.Messages.getString("gnu_gpl")+"<br>"
+			+ lang.Messages.getString("copyright")+"<br>"
+			+ lang.Messages.getString("url_text")+"<br>"
+			+ lang.Messages.getString("emailaddress")+"<br>"
+			+"<br>"
+			+ lang.Messages.getString("thanks_to")+"<br><ul><li>"
+			//x.oswald@free.fr
+			//y.durand@gmx.de
+			+ lang.Messages.getString("french_translators")+"</li><li>"
+			//pablo.pita@pitagoral.com
+			+ lang.Messages.getString("spanish_translators")+"</li><li>"
+			//gudrungeorges@gmail.com
+			+ lang.Messages.getString("english_translators")+"</li><li>"
+			//Loupiotebec@aol.com
+			+ lang.Messages.getString("italian_translators")+"</li><li>"
+			//khemis@bol.com.br
+			+ lang.Messages.getString("portuguese_translators")+"</li><li>"
+			//cazaria@gmx.net
+	//		+ lang.Messages.getString("polish_translators")+"</li><li>"
+			//jredrejo@edu.juntaextremadura.net>
+			+ lang.Messages.getString("debian_package_maintainer")+"</li><li>"
+			+ lang.Messages.getString("icon_designer")+"</li></ul>"
+		;//String
+	}//Constructor
 	
 	/**
 	 * 
@@ -411,9 +447,9 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		}else if (prog == "compareFractions"){
 			points_compareFractions = p;
 			dsp_compareFractions.setText(String.valueOf(p));
-		}else if (prog == "expandFraction"){
-			points_expandFraction = p;
-			dsp_expandFraction.setText(String.valueOf(p));
+		}else if (prog == "extendFraction"){
+			points_extendFraction = p;
+			dsp_extendFraction.setText(String.valueOf(p));
 		}else if (prog == "reduceFraction"){
 			points_reduceFraction = p;
 			dsp_reduceFraction.setText(String.valueOf(p));
@@ -458,6 +494,16 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		jmiItalian.setText(lang.Messages.getString("italian"));
 		jmiPortuguese.setText(lang.Messages.getString("portuguese"));
 //		jmiPolish.setText(lang.Messages.getString("polish"));
+		jmWorkSheet.setText(lang.Messages.getString("worksheets"));
+//		jmiExtendingWorkSheet.setText(lang.Messages.getString("extend_fractions"));
+//		jmiReducingWorkSheet.setText(lang.Messages.getString("reduce_fractions"));
+//		jmiImproperFractionsWorkSheet.setText(lang.Messages.getString("improper_fracs"));
+//		jmiMixedNumbersWorkSheet.setText(lang.Messages.getString("mixed_numbers"));
+		jmiAdditionWorkSheet.setText(lang.Messages.getString("add_fractions"));
+//		jmiSubtractionWorkSheet.setText(lang.Messages.getString("minus_fractions"));
+		jmiMultiplicationWorkSheet.setText(lang.Messages.getString("multiply_fractions"));
+//		jmiDivisionWorkSheet.setText(lang.Messages.getString(""));//TODO new snippet or three kinds of worksheets
+
 		jmHelp.setText(lang.Messages.getString("Info"));
 		jmiHelp.setText(lang.Messages.getString("jmi_help"));
 		cb_showtippatstart.setText(lang.Messages.getString("show_hints_on_start"));
@@ -467,7 +513,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		btn_clickNumerator.setText(lang.Messages.getString("click_numerator"));
 		btn_nameFraction.setText(lang.Messages.getString("name_fractions"));
 		btn_compareFractions.setText(lang.Messages.getString("compare_fractions"));
-		btn_expandFraction.setText(lang.Messages.getString("extend_fractions"));
+		btn_extendFraction.setText(lang.Messages.getString("extend_fractions"));
 		btn_reduceFraction.setText(lang.Messages.getString("reduce_fractions"));
 		btn_improperFracions.setText(lang.Messages.getString("improper_fracs"));
 		btn_mixedNumbers.setText(lang.Messages.getString("mixed_numbers"));
@@ -515,7 +561,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
         		new NameFraction(this,location_x, location_y, size_x-130, size_y+150);
         	}else if (obj == btn_compareFractions){
         		new CompareFractions(this,location_x, location_y, size_x-50, size_y+200);
-        	}else if (obj == btn_expandFraction){
+        	}else if (obj == btn_extendFraction){
         		new ExtendFraction(this,location_x, location_y, size_x, size_y+200);
         	}else if (obj == btn_reduceFraction){
         		new ReduceFraction(this,location_x, location_y, size_x, size_y+200);
@@ -540,7 +586,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
         			points_clickNumerator,
         			points_nameFraction,
         			points_compareFractions,
-        			points_expandFraction,
+        			points_extendFraction,
         			points_reduceFraction,
         			points_unechteBrueche,
         			points_gemischteZahlen,
@@ -555,7 +601,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 	        		setPoints(0, "clickNumerator");
 	    			setPoints(0, "nameFraction");
 	    			setPoints(0, "compareFractions");
-	    			setPoints(0, "expandFraction");
+	    			setPoints(0, "extendFraction");
 	    			setPoints(0, "reduceFraction");
 	    			setPoints(0, "unechteBrueche");
 	    			setPoints(0, "gemischteZahlen");
@@ -569,7 +615,7 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 		}else if (obj == btn_readTheLog){
 			JOptionPane.showMessageDialog(
 				null,
-				lang.Messages.getString("choose_dir")
+				lang.Messages.getString("choose_logdir")
 			);
 			JFileChooser fc = new JFileChooser();
 			ResultOfGame sl = new ResultOfGame();
@@ -579,12 +625,6 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 			int returnVal = fc.showOpenDialog(this);
 			if (returnVal == JFileChooser.APPROVE_OPTION) {
 				File file = fc.getSelectedFile();
-//				FileFilter myfilter = new FileNameExtensionFilter("jfl", "JFL");
-//				fc.setFileFilter(myfilter);
-//				does not work,
-//				because fc is set to JFileChooser.DIRECTORIES_ONLY
-//				so fc ignores files!
-//				use MyFilenameFilter !
 				String[] dir = file.list(new MyFilenameFilter("jfl"));
 				Object[][] data = new Object[dir.length][nbOfCols];
 				java.util.Arrays.sort(dir);
@@ -613,16 +653,14 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 						ex.printStackTrace();
 						JOptionPane.showMessageDialog(
 							null,
-							lang.Messages.getString("file_could_not_be_opened")
+							lang.Messages.getString("file_can_not_be_opened")
 						);
 					}//acatch
 				}//for
-				System.out.println("readTheLog");
 				for (int as=0; as<dir.length; as++){
 					for(int is=0; is<nbOfCols; is++){
 						System.out.print("data:"+data[as][is]);
 					}
-					System.out.println("");
 				}//for
 				new LogTable(this, data);
 			}//returnVal
@@ -648,6 +686,33 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 //		else if (obj == jmiPolish){
 //			lang.Messages.setLocale(new Locale("pl"));
 //			refreshCaption();
+//		}
+//		else if(obj == jmiExtendingWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
+//		}else if(obj == jmiReducingWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
+//		}else if(obj == jmiImproperFractionsWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
+//		}else if(obj == jmiMixedNumbersWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
+//		}
+		else if(obj == jmiAdditionWorkSheet){
+			new AdditionWorkSheetDialog();
+		}
+//		else if(obj == jmiSubtractionWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
+//		}
+		else if(obj == jmiMultiplicationWorkSheet){
+			new MultiplicationWorksheetDialog();
+		}
+//		else if(obj == jmiDivisionWorkSheet){
+//			//TODO implement this worksheet
+//			System.out.println("not yet implemented");
 //		}
 		else if (obj == jmiHelp){
 			new HelpStarter(
@@ -655,7 +720,20 @@ public class JFractionLab extends JFrame implements ActionListener, ItemListener
 					"index"
 			);
 		}else if (obj == jmiAbout){
-			new InfoDialog();
+			new InfoDialog(lang.Messages.getString("about_jfl"), aboutText);
 		}//if else
 	}//actionPerformed
+	
+	public static int ggt(int a, int b){
+		// verwendet wird der erweiterte Euklidsche Algorithmus
+		if(a==b||b==0) return a;
+		else return ggt(b,a%b);
+	}
+
+	public static int kgv(int a, int b){
+		// verwendet wird der Satz ggT(a,b) * kgV(a,b) = a*b
+		return (a*b)/ggt(a,b);
+	}
+	
+	
 }//class
